

void SetDspeed()                                          // set pen servo to raised position
{
  volts=analogRead(Battery)/10;
  
  if(volts<lowbat || pause!=-1)                    // once voltage falls below lowbat value
  {
    lspeed=0;                                      // stop left  motor
    rspeed=0;                                      // stop right motor
    Motors();                                      // update motor control pins
    if (pause<0) Up();                             // make sure pen is raised when battery first falls below lowbat value
    pause++;                                       // increment pause
    if(pause>1) pause=0;                           // toggle pause between 0 and 1 once battery is flat
    digitalWrite(13,pause);                        // flash LED on D13
    delay(200);                                    // set flashing LED speed
    return;                                        // reset loop
  }
                                                   // current is proportional to voltage and power = voltage x current
                                                   // As the battery voltage drops dspeed increases to maintain a constant speed
  
  dspeed=26214/volts*10/volts;                     // draw speed power correction factor=100% when battery=5V (analog reading = 512)
  dspeed=dspeed*8/10;                              // adjust speed to reduce overshoot
}

void Calibrate()                                   // Calibrate diffSpeed's
{
  Down();
  int distance = 40;
  diffSpeedF = CalibrateDiffSpeed( +1, +1, distance);
//  delay(2000);
  diffSpeedB = CalibrateDiffSpeed( -1, -1, distance);
//  delay(2000);
  diffSpeedL = CalibrateDiffSpeed( -1, +1, distance);
//  delay(2000);
  diffSpeedR = CalibrateDiffSpeed( +1, -1, distance);
//  delay(2000);
  
  tickTimeF = MeasureTickTime( +1, +1, distance);
//  delay(2000);
  tickTimeB = MeasureTickTime( -1, -1, distance);
//  delay(2000);
  tickTimeL = MeasureTickTime( -1, +1, distance);
//  delay(2000);
  tickTimeR = MeasureTickTime( +1, -1, distance);
  delay(2000);
  
  Up();
}


// Calculate speed difference between wheels ("diffSpeed").
// Value >0 means left wheel is faster than the right and its speed needs to be reduced.
//
int CalibrateDiffSpeed(int lWheelDir, int rWheelDir, int ticks) {
  
  Stop();
 
  SetDspeed();
  int origspeed=dspeed;
  if (lWheelDir*rWheelDir<0) 					   // Use slower speed when turning. 
	dspeed=dspeed*turnFactor;
  
  // move left wheel to the next tick
  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  lspeed=lWheelDir*dspeed;
  rspeed=0;

  while(lcount<1)        // travel until tick is encountered
  {
    Motors();
  }
  Stop();                                          // ensure both motors are completely stopped

  // move left wheel to the next tick
  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  lspeed=0;
  rspeed=rWheelDir*dspeed;

  while(rcount<1)        // travel until tick is encountered
  {
    Motors();
  }
  Stop();                                          // ensure both motors are completely stopped

  delay(500);

  // move both wheels for calibration
  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  lspeed=lWheelDir*dspeed;
  rspeed=rWheelDir*dspeed;

  while(lcount<ticks || rcount<ticks)        // travel until both wheel encoders count correct distance
  {
    Motors();
  }
  Stop();                                          // ensure both motors are completely stopped

  int ret= dspeed*((float)(lcount-rcount)/(float)max(lcount,rcount));
  
  dspeed=origspeed;
  
  return ret;

}


// Measure the time per tick
//
int MeasureTickTime(int lWheelDir, int rWheelDir, int ticks) {
  
  Stop();
 
  SetDspeed();

  int origspeed=dspeed;
  if (lWheelDir*rWheelDir<0)						// Use slower speed when turning. 
	dspeed=dspeed*turnFactor;
  

  // move left wheel to the next tick
  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  lspeed=lWheelDir*dspeed;
  rspeed=0;

  while(lcount<1)        // travel until tick is encountered
  {
    Motors();
  }
  Stop();                                          // ensure both motors are completely stopped

  // move left wheel to the next tick
  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  lspeed=0;
  rspeed=rWheelDir*dspeed;

  while(rcount<1)        // travel until tick is encountered
  {
    Motors();
  }
  Stop();                                          // ensure both motors are completely stopped

  delay(500);

  // move both wheels for calibration
  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  lspeed=lWheelDir*dspeed;
  rspeed=rWheelDir*dspeed;

  // adjust speed difference
  int diffSpeed;
  if (lWheelDir > 0)
  {
    if (rWheelDir > 0)
    {
      diffSpeed = diffSpeedF;
    }
    else
    {
      diffSpeed = diffSpeedR;
    }
  }
  else
  {
    if (rWheelDir > 0)
    {
      diffSpeed = diffSpeedL;
    }
    else
    {
      diffSpeed = diffSpeedB;
    }
  }

  if (diffSpeed > 0) 
  {
    lspeed -= lWheelDir*diffSpeed;
  }
  else
  {
    rspeed -= rWheelDir*diffSpeed;
  }

  int startTime=millis();
  while(lcount<ticks || rcount<ticks)        // travel until both wheel encoders count correct distance
  {
    Motors();
  }
  int endTime=millis();
  Stop();                                          // ensure both motors are completely stopped

  dspeed=origspeed;

  return ((endTime-startTime)/(float)ticks);
}

