#include <EEPROM.h>

#include "Constants.h"
#include "IOpins.h"

#include "EEPROMAnything.h"

int volts;                                         // battery voltage
int pause=-1;                                      // a value of -1 means the battery voltage is ok, 0 and 1 are used to flash LED on D13 when the battery is flat
byte pen;
byte turn;
//int count=0;								   // How many times the robot was run

int lspeed;                                    // left  speed -255 to +255 (negative values are reverse)
int rspeed;                                    // right speed -255 to +255 (negative values are reverse)
int dspeed=80;                                 // draw  speed  +50 to +200

int diffSpeedF = 0;                            // Forward - difference of speeds between left and right - positive: left should be decremented
int diffSpeedB = 0;                            // Backward - difference of speeds between left and right - positive: left should be decremented
int diffSpeedL = 0;                            // Left turn - difference of speeds between left and right - positive: left should be decremented
int diffSpeedR = 0;                            // Right turn - difference of speeds between left and right - positive: left should be decremented

int tickTimeF = 0;                             // Forward - time to travel one tick
int tickTimeB = 0;                             // Backward - time to travel one tick
int tickTimeL = 0;                             // Left turn - time to travel one tick
int tickTimeR = 0;                             // Right turn - time to travel one tick

volatile int rcount;
volatile int lcount;

void setup()
{
  attachInterrupt(0,Rcount,CHANGE);
  attachInterrupt(1,Lcount,CHANGE);
  
  Serial.begin(57600);                             // initialize serial interface - 9600 baud   
  pinMode(Ldirpin,OUTPUT);                         // set left  direction pin as an output pin
  pinMode(Rdirpin,OUTPUT);                         // set right direction pin as an output pin
  pinMode(Srvopin,OUTPUT);                         // set servo  control  pin as an output pin
  
  digitalWrite(2,1);                               // enable pullup resistor on D2
  digitalWrite(3,1);                               // enable pullup resistor on D3
  
  Up();                                            // start the program with the pen raised  
  
  //--------------------------------------------------------------------

  pinMode(7,OUTPUT);                                                         // left  motor direction           
  pinMode(8,OUTPUT);                                                         // right motor direction
  pinMode(9,OUTPUT);                                                         // left  motor PWM
  pinMode(10,OUTPUT);                                                        // right motor PWM
  pinMode(13,OUTPUT);

  readDataFromEEPROM();  				// Read calibration data from the previous run.
  
}

void loop()
{ 
  
  // !!!!!!!!!
  // !!! Important: Full calibration code, plus Robot drawing is too much for a sketch size.
  // !!!  Run the calibration code, then comment it and un-comment 'drawRobot' and re-run the robot.
  // !!!!!!!!!
  SetDspeed();
  Calibrate();
  writeDataToEEPROM();  				// Save calibration data for next run.
  
  delay(2000);
   
  // drawRobot();							// Do what we came here to do!   

  END();
}

// Draw the robot.
//
void drawRobot() {
    int factor=3; 				// How big the robot should be?
		
    float instructions[] = {
    3,-90,
	//Right hand
    2,90,1.2,90,3.5,
	//Neck
    -90,1.1,
	//Head
    -90,1.1,90,2,90,3.5,90,2,90,1.1,
	//Neck
    -90,1.1,
	//Left Hand
    -90,3.5,90,1.2,80,2,-90,3,90,1,90,2,-90,1.5,-90,2,90,1,0};
    int numInstructions = 40;

	Down();  
    int i = 0;
    while (i < numInstructions)
    {
	  Go(instructions[i++]*factor);
      //i++;
   	  Turn(instructions[i++]);
      //i++;
    }
}



// Write some values to EEPROM to be read next time.
//
void writeDataToEEPROM() {
  int address=0;
  address+=EEPROM_writeAnything(address, volts);
  address+=EEPROM_writeAnything(address, dspeed);
  address+=EEPROM_writeAnything(address, diffSpeedF);
  address+=EEPROM_writeAnything(address, diffSpeedB);
  address+=EEPROM_writeAnything(address, diffSpeedL);
  address+=EEPROM_writeAnything(address, diffSpeedR);
  address+=EEPROM_writeAnything(address, tickTimeF);
  address+=EEPROM_writeAnything(address, tickTimeB);
  address+=EEPROM_writeAnything(address, tickTimeL);
  address+=EEPROM_writeAnything(address, tickTimeR);
}

// Read stored data from EEPROM.
// Can be used to skip calibration phase or as a debugging tool.
// 
void readDataFromEEPROM() {
 
  int address=0;
  address+=EEPROM_readAnything(address, volts);
  address+=EEPROM_readAnything(address, dspeed);
  address+=EEPROM_readAnything(address, diffSpeedF);
  address+=EEPROM_readAnything(address, diffSpeedB);
  address+=EEPROM_readAnything(address, diffSpeedL);
  address+=EEPROM_readAnything(address, diffSpeedR);
  address+=EEPROM_readAnything(address, tickTimeF);
  address+=EEPROM_readAnything(address, tickTimeB);
  address+=EEPROM_readAnything(address, tickTimeL);
  address+=EEPROM_readAnything(address, tickTimeR);
//  address+=EEPROM_readAnything(address, count);
}

void END()
{
  while(1<2) {}
}

  





