// Go straight line to a specific distance.
// The distance is measured in "ticks": The full round of wheel is divided to 8 ticks by the encoder.
// However, the method can handle also "parts of tick", so the distance parameter is of type 'float'.
//
void Go(float distance)
{
  int travelTime = 0;

  lcount=0;                                        // reset left wheel encoder counter
  rcount=0;                                        // reset right wheel encoder counter
  
  Stop();											// just in case.
  delay(100);
  
  SetDspeed();

  float ticks = abs(distance);

  if(distance>0)                                   // distance values>0 tell the robot to go forward
  {
    travelTime = tickTimeF*ticks;
    lspeed=dspeed;
    rspeed=dspeed;
  }
  else                                             // distance values less than 0 tell the robot to go backward
  {
    travelTime = tickTimeF*ticks;
    lspeed=-dspeed;
    rspeed=-dspeed;
  }

  distance=abs(distance);                          // distance only - ignore direction
  if(turn==0) 
  {
    distance*=charsize;
    travelTime*=charsize;
  }
  int ols=lspeed;
  int ors=rspeed;

  static int tempDiff[2] = {0,0};
  boolean dir=distance>0;
  
  int startTime=millis();
  while (true) {
    int curTime=millis();
    if (curTime-startTime>=travelTime) 
		break;
   
      
    if (lcount<ticks-1 ) {
		if(lcount-rcount > 1)
		{
			if (lspeed != 0) tempDiff[dir] += (lcount-rcount) / 2;
			lspeed=0;
		}
		else
		{
			if (tempDiff[dir] > 0) 
			{
				lspeed=(ols > 0) ? ols - tempDiff[dir] : ols + tempDiff[dir];
			}
			else
			{
				lspeed=ols;
			}
		}
    }
    
    if (rcount<ticks-1 ) {
      
		if(rcount-lcount > 1)
		{
			if (rspeed != 0) tempDiff[dir] += (lcount-rcount) / 2;
			rspeed=0;
		}
		else
		{
			if (tempDiff[dir] < 0) 
			{
				rspeed=(ors > 0) ? ors - -tempDiff[dir] : ors + -tempDiff[dir];
			}
			else
			{
				rspeed=ors;
			}
		}
    }
    Motors();
  }
  
  //Handle OverShoot!!!
  lspeed=-ols;
  rspeed=0;
  Motors();
  delay(2);  
  
  Stop();                                          // ensure both motors are completely stopped
    
}
