// Turn for specific angle. 
// Similar to the 'Go' routine, the angle is of type 'float'. 
//
void Turn(float angle)                               // +angle=CW, -angle=CCW
{
  int travelTime = 0;

  byte oldpen=pen;
  float jump=4.7;
  
  Up();
  turn=1;
  Go(jump);
  turn=0;

  int origspeed=dspeed;
  dspeed=dspeed*turnFactor;

  lcount=0;
  rcount=0;
  
//  float ticks = abs(angle) / 22.5;        // calculated using: wheel diameter 4 cm, wheels distance: 8 cm
  float ticks = abs(angle) / 21.635;        // calculated using: wheel diameter 3.75 cm, wheels distance: 7.8 cm

  if(angle<0)
  {
    travelTime = tickTimeL*ticks;
    lspeed=-dspeed;
    rspeed=dspeed;
    if (diffSpeedL > 0) 
    {
      lspeed += diffSpeedL;
    }
    else
    {
      rspeed += diffSpeedL;
    }
  }
  else
  {
    travelTime = tickTimeR*ticks;
    lspeed=dspeed;
    rspeed=-dspeed;
    if (diffSpeedR > 0) 
    {
      lspeed -= diffSpeedR;
    }
    else
    {
      rspeed -= diffSpeedR;
    }
  }
  int ols=lspeed;
  int ors=rspeed;

  travelTime+=50; 		// Handle slow starting time. Note: This value seems to be dependant of battery voltage.
    
  static int tempTurnDiff[2] = {0,0};
  boolean dir=angle>0;
    
  int startTime=millis();
  while(true) {
    int curTime=millis();
    if (curTime-startTime>=travelTime) 
		break;
        
    if (lcount<ticks-1 ) 
    {
        if(lcount-rcount > 1)
        {
            if (lspeed != 0) tempTurnDiff[dir] += (lcount-rcount) / 2;
            lspeed=0;
        }
        else
        {
            if (tempTurnDiff[dir] > 0) 
            {
                lspeed=(ols > 0) ? ols - tempTurnDiff[dir] : ols + tempTurnDiff[dir];
            }
            else
            {
                lspeed=ols;
            }
         }
     }
    
    if (rcount<ticks-1 ) {
      
        if(rcount-lcount > 1)
        {
            if (rspeed != 0) tempTurnDiff[dir] += (lcount-rcount) / 2;
            rspeed=0;
        }
        else
        {
            if (tempTurnDiff[dir] < 0) 
            {
                rspeed=(ors > 0) ? ors - -tempTurnDiff[dir] : ors + -tempTurnDiff[dir];
            }
            else
            {
                rspeed=ors;
            }
        }
        Motors();
    }
        
        
    }//while
	
	// Handle overshoot!
	lspeed=-ols;
    rspeed=0;
    Motors();
    delay(5);  
    
	Stop();                                          // ensure both motors are completely stopped
    
  dspeed=origspeed;

  turn=1;
  Go(-jump-1.1);
  turn=0;
  if(oldpen==1) Down();

}



